==> u%2B%2B_5.0.1.orig.tar.gz: u++-5.0.1/src/examples/RWEx2.cc <==
//                              -*- Mode: C++ -*- 
// 
// uC++ Version 5.0.1, Copyright (C) Peter A. Buhr 1994
// 
// RWEx2.cc -- Readers and Writer Problem
// 
// Author           : Peter A. Buhr
// Created On       : Thu Aug  2 11:51:34 1990
// Last Modified By : Peter A. Buhr
// Last Modified On : Thu Jan 29 08:32:30 2004
// Update Count     : 71
// 

#include <uC++.h>
#include <uIOStream.h>


uMonitor ReaderWriter {
	int ReadCount, WriteUsage;
	uCondition ReaderAndWriter;
	enum RW { READER, WRITER };
  public:
	ReaderWriter() {
		ReadCount = WriteUsage = 0;
	} // ReaderWriter

	void StartRead() {
		if ( WriteUsage || !ReaderAndWriter.uEmpty() ) {
			uWait ReaderAndWriter uWith READER;
		} // if
		ReadCount += 1;
	} // ReaderWriter::StartRead

	void EndRead() {
		ReadCount -= 1;
		if ( ReadCount == 0 && !ReaderAndWriter.uEmpty() ) {
			uSignal ReaderAndWriter;
		} // if
	} // ReaderWriter::EndRead

	void StartWrite() {
		if ( WriteUsage || ReadCount != 0 ) {
			uWait ReaderAndWriter uWith WRITER;
		} // if
		WriteUsage = 1;
	} // ReaderWriter::StartWrite

	void EndWrite() {
		WriteUsage = 0;
		if ( !ReaderAndWriter.uEmpty() ) {
			if ( ReaderAndWriter.uFront() == WRITER ) { // writer at front of queue ?
				uSignal ReaderAndWriter;
			} else {									// release readers at front of queue
				for ( ;; ) {
					uSignal ReaderAndWriter;
				  if ( ReaderAndWriter.uEmpty() ) break;
				  if ( ReaderAndWriter.uFront() == WRITER ) break; // writer at front of queue ?
				} // for
			} // if
		} // if
	} // ReaderWriter::EndWrite
}; // ReaderWriter


volatile int SharedVar = 0;								// shared variable to test readers and writers

uTask Worker {
	ReaderWriter &rw;

	void main() {
		uYield( rand() % 100 );							// don't all start at the same time
		if ( rand() % 100 < 70 ) {						// decide to be a reader or writer
			rw.StartRead();
			uCout << uAcquire << "Reader:" << this << ", shared:" << SharedVar << endl << uRelease;
			uYield( 3 );
			rw.EndRead();
		} else {
			rw.StartWrite();
			SharedVar += 1;
			uCout << uAcquire << "Writer:" << this << ",  wrote:" << SharedVar << endl << uRelease;
			uYield( 1 );
			rw.EndWrite();
		} // if
	} // Worker::main
  public:
	Worker( ReaderWriter &rw ) : rw( rw ) {
	} // Worker::Worker
}; // Worker

#define MaxTask 50

void uMain::main() {
	ReaderWriter rw;
	Worker *workers;
	
	workers = new Worker[MaxTask]( rw );
	delete [] workers;

	uCout << uAcquire << "successful completion" << endl << uRelease;
} // uMain::main


// Local Variables: //
// tab-width: 4 //
// compile-command: "u++ RWEx2.cc" //
// End: //

==> u%2B%2B_5.3.0.orig.tar.gz: u++-5.3.0/src/examples/RWEx2.cc <==
//                              -*- Mode: C++ -*- 
// 
// uC++ Version 5.3.0, Copyright (C) Peter A. Buhr 1994
// 
// RWEx2.cc -- Readers and Writer Problem
// 
// Author           : Peter A. Buhr
// Created On       : Thu Aug  2 11:51:34 1990
// Last Modified By : Peter A. Buhr
// Last Modified On : Wed Nov 30 08:41:51 2005
// Update Count     : 79
// 

#include <uC++.h>
#include <iostream>
using std::cout;
using std::osacquire;
using std::endl;

_Monitor ReaderWriter {
	int ReadCount, WriteUsage;
	uCondition ReaderAndWriter;
	enum RW { READER, WRITER };
  public:
	ReaderWriter() {
		ReadCount = WriteUsage = 0;
	} // ReaderWriter

	void StartRead() {
		if ( WriteUsage || !ReaderAndWriter.empty() ) {
			ReaderAndWriter.wait( READER );
		} // if
		ReadCount += 1;
	} // ReaderWriter::StartRead

	void EndRead() {
		ReadCount -= 1;
		if ( ReadCount == 0 && !ReaderAndWriter.empty() ) {
			ReaderAndWriter.signal();
		} // if
	} // ReaderWriter::EndRead

	void StartWrite() {
		if ( WriteUsage || ReadCount != 0 ) {
			ReaderAndWriter.wait( WRITER );
		} // if
		WriteUsage = 1;
	} // ReaderWriter::StartWrite

	void EndWrite() {
		WriteUsage = 0;
		if ( !ReaderAndWriter.empty() ) {
			if ( ReaderAndWriter.uFront() == WRITER ) { // writer at front of queue ?
				ReaderAndWriter.signal();
			} else {									// release readers at front of queue
				for ( ;; ) {
					ReaderAndWriter.signal();
				  if ( ReaderAndWriter.empty() ) break;
				  if ( ReaderAndWriter.uFront() == WRITER ) break; // writer at front of queue ?
				} // for
			} // if
		} // if
	} // ReaderWriter::EndWrite
}; // ReaderWriter


volatile int SharedVar = 0;								// shared variable to test readers and writers

_Task Worker {
	ReaderWriter &rw;

	void main() {
		yield( rand() % 100 );							// don't all start at the same time
		if ( rand() % 100 < 70 ) {						// decide to be a reader or writer
			rw.StartRead();
			osacquire( cout ) << "Reader:" << this << ", shared:" << SharedVar << endl;
			yield( 3 );
			rw.EndRead();
		} else {
			rw.StartWrite();
			SharedVar += 1;
			osacquire( cout ) << "Writer:" << this << ",  wrote:" << SharedVar << endl;
			yield( 1 );
			rw.EndWrite();
		} // if
	} // Worker::main
  public:
	Worker( ReaderWriter &rw ) : rw( rw ) {
	} // Worker::Worker
}; // Worker

#define MaxTask 50

void uMain::main() {
	ReaderWriter rw;
	Worker *workers;
	
	workers = new Worker[MaxTask]( rw );
	delete [] workers;

	osacquire( cout ) << "successful completion" << endl;
} // uMain::main


// Local Variables: //
// tab-width: 4 //
// compile-command: "u++ RWEx2.cc" //
// End: //
